@@ -1,6 +1,8 @@
-from rest_framework import permissions
-from rest_framework import filters
 from guardian.shortcuts import get_objects_for_user
+from rest_framework import filters
+from rest_framework import permissions
+
+from djangoldp.models import Model
 
 """
 Liste des actions passées dans views selon le protocole REST :
@@ -16,6 +18,7 @@ Pour chacune de ces actions, on va définir si on accepte la requête (True) ou
     checks have already passed
 """
 
+
 class WACPermissions(permissions.DjangoObjectPermissions):
     perms_map = {
         'GET': ['%(app_label)s.view_%(model_name)s'],
@@ -33,6 +36,13 @@ class WACPermissions(permissions.DjangoObjectPermissions):
         else:
             return super().has_permission(request, view)
 
+    # This method should be overriden by other permission classes
+    def user_permissions(self, user, obj):
+        return []
+
+    def filter_user_perms(self, user_or_group, obj, permissions):
+        return [perm for perm in permissions if perm in self.user_permissions(user_or_group, obj)]
+
 
 class ObjectFilter(filters.BaseFilterBackend):
     def filter_queryset(self, request, queryset, view):
@@ -43,29 +53,41 @@ class ObjectFilter(filters.BaseFilterBackend):
         objects = get_objects_for_user(request.user, perm, klass=queryset)
         return objects
 
+
 class ObjectPermission(WACPermissions):
     filter_class = ObjectFilter
 
+
 class InboxPermissions(WACPermissions):
     """
-        Anonymous users: can create notifications but can't read
-        Logged in users: can create notifications but can't read
-        Inbox owners: can read + update all notifications
+        Everybody can create
+        Author can edit
     """
-    filter_class = ObjectFilter
+    anonymous_perms = ['view', 'create']
+    authenticated_perms = ['view', 'create']
+    author_perms = ['view']
+
     def has_permission(self, request, view):
-        if view.action in ['create', 'retrieve', 'update', 'partial_update', 'destroy']:
+        if view.action in ['create', 'list', 'retrieve']:
             return True
         else:
             return super().has_permission(request, view)
 
     def has_object_permission(self, request, view, obj):
-        if view.action == "create":
-            return True
-        if hasattr(obj._meta, 'auto_author'):
-            if request.user == getattr(obj, obj._meta.auto_author):
-                return True
-        return super().has_object_permission(request, view)
+        if view.action in ['update', 'partial_update', 'destroy']:
+            return False
+        else:
+            return super().has_object_permission(request, view)
+
+    def user_permissions(self, user, obj):
+        if user.is_anonymous:
+            return self.anonymous_perms
+        else:
+            if Model.get_meta(obj, 'auto_author') == user:
+                return self.author_perms
+            else:
+                return self.authenticated_perms
+
 
 class AnonymousReadOnly(WACPermissions):
     """
@@ -74,20 +96,22 @@ class AnonymousReadOnly(WACPermissions):
         Author: can read all posts + create new posts + update their own
     """
 
-    anonymous_perms = [{'mode': {'@type': 'view'}}]
-    authenticated_perms = [{'mode': {'@type': 'view'}}, {'mode': {'@type': 'add'}}]
-    author_perms = [{'mode': {'@type': 'view'}}, {'mode': {'@type': 'add'}}, {'mode': {'@type': 'change'}}]
+    anonymous_perms = ['view']
+    authenticated_perms = ['view', 'add']
+    author_perms = ['view', 'add', 'change', 'control', 'delete']
 
     def has_permission(self, request, view):
         if view.action in ['list', 'retrieve']:
             return True
+        elif view.action == 'create' and request.user.is_authenticated():
+            return True
         else:
             return super().has_permission(request, view)
 
     def has_object_permission(self, request, view, obj):
         if view.action == "create" and request.user.is_authenticated():
             return True
-        elif view.action == "retrieve":
+        elif view.action in ["list", "retrieve"]:
             return True
         elif view.action in ['update', 'partial_update', 'destroy']:
             if hasattr(obj._meta, 'auto_author'):
@@ -95,4 +119,13 @@ class AnonymousReadOnly(WACPermissions):
                 if author == request.user:
                     return True
         else:
-            return super().has_object_permission(request, view, obj)
\ No newline at end of file
+            return super().has_object_permission(request, view, obj)
+
+    def user_permissions(self, user, obj):
+        if user.is_anonymous:
+            return self.anonymous_perms
+        else:
+            if Model.get_meta(obj, 'auto_author') == user:
+                return self.author_perms
+            else:
+                return self.authenticated_perms
