@@ -6,7 +6,6 @@ from django.core.exceptions import ValidationError as DjangoValidationError
 from django.core.urlresolvers import get_resolver, resolve, get_script_prefix, Resolver404
 from django.utils.datastructures import MultiValueDictKeyError
 from django.utils.encoding import uri_to_iri
-from guardian.shortcuts import get_perms
 from rest_framework.exceptions import ValidationError
 from rest_framework.fields import SkipField, empty
 from rest_framework.fields import get_error_detail, set_value
@@ -17,7 +16,6 @@ from rest_framework.utils import model_meta
 from rest_framework.utils.field_mapping import get_nested_relation_kwargs
 from rest_framework.utils.serializer_helpers import ReturnDict
 
-from djangoldp import permissions
 from djangoldp.fields import LDPUrlField, IdURLField
 from djangoldp.models import Model
 
@@ -35,7 +33,11 @@ class LDListMixin:
         return [self.child.to_internal_value(item) for item in data]
 
     def to_representation(self, value):
-        return {'@id': self.id, '@type': 'ldp:Container', 'ldp:contains': super().to_representation(value)}
+        return {'@id': self.id,
+                '@type': 'ldp:Container',
+                'ldp:contains': super().to_representation(value),
+                'permissions': Model.get_permissions(value.model, self.context['request'].user, ['view', 'add'])
+                }
 
     def get_attribute(self, instance):
         parent_id_field = self.parent.fields[self.parent.url_field_name]
@@ -59,7 +61,7 @@ class LDListMixin:
             try:
                 list = next(
                     filter(lambda o: list[self.parent.url_field_name] == o[self.parent.url_field_name], object_list))
-            except (KeyError, TypeError):
+            except (KeyError, TypeError, StopIteration):
                 pass
 
             try:
@@ -196,35 +198,34 @@ class LDPSerializer(HyperlinkedModelSerializer):
             fields = list(self.Meta.model._meta.serializer_fields)
         except AttributeError:
             fields = super().get_default_field_names(declared_fields, model_info)
-        try:
-            fields.remove(self.Meta.model._meta.auto_author)
-        except ValueError:
-            pass
-        except AttributeError:
-            pass
+        if 'request' in self._context and not self._context['request']._request.method == 'GET':
+            try:
+                fields.remove(self.Meta.model._meta.auto_author)
+            except ValueError:
+                pass
+            except AttributeError:
+                pass
         return fields + list(getattr(self.Meta, 'extra_fields', []))
 
     def to_representation(self, obj):
         data = super().to_representation(obj)
 
-        if hasattr(obj._meta, 'rdf_type'):
-            data['@type'] = obj._meta.rdf_type
-        data['permissions'] = [{'mode': {'@type': name.split('_')[0]}} for name in
-                               get_perms(self.context['request'].user, obj)]
-
-        if self.context['request'].user.is_anonymous:
-            data['permissions'] += permissions.AnonymousReadOnly.anonymous_perms
-        elif self.context['request'].user.is_authenticated:
-            if hasattr(obj._meta, 'auto_author'):
-                data['permissions'] += permissions.AnonymousReadOnly.author_perms
-            else:
-                data['permissions'] += permissions.AnonymousReadOnly.authenticated_perms
-
-        if hasattr(obj._meta, 'rdf_context'):
-            data['@context'] = obj._meta.rdf_context
+        rdf_type = Model.get_meta(obj, 'rdf_type', None)
+        rdf_context = Model.get_meta(obj, 'rdf_context', None)
+        if rdf_type is not None:
+            data['@type'] = rdf_type
+        if rdf_context is not None:
+            data['@context'] = rdf_context
+        data['permissions'] = Model.get_permissions(obj, self.context['request'].user,
+                                                    ['view', 'change', 'control', 'delete'])
 
         return data
 
+    def build_field(self, field_name, info, model_class, nested_depth):
+        nested_depth = self.compute_depth(nested_depth, model_class)
+
+        return super().build_field(field_name, info, model_class, nested_depth)
+
     def build_standard_field(self, field_name, model_field):
         class JSonLDStandardField:
             parent_view_name = None
@@ -253,6 +254,8 @@ class LDPSerializer(HyperlinkedModelSerializer):
         return type(field_class.__name__ + 'Valued', (JSonLDStandardField, field_class), {}), field_kwargs
 
     def build_nested_field(self, field_name, relation_info, nested_depth):
+        nested_depth = self.compute_depth(nested_depth, self.Meta.model)
+
         class NestedLDPSerializer(self.__class__):
 
             class Meta:
@@ -307,7 +310,8 @@ class LDPSerializer(HyperlinkedModelSerializer):
 
                     try:
                         match = resolve(uri_to_iri(uri))
-                        ret['pk'] = match.kwargs['pk']
+                        slug_field = Model.slug_field(self.__class__.Meta.model)
+                        ret[slug_field] = match.kwargs[slug_field]
                     except Resolver404:
                         pass
 
@@ -320,11 +324,21 @@ class LDPSerializer(HyperlinkedModelSerializer):
         kwargs['required'] = False
         return NestedLDPSerializer, kwargs
 
+    @classmethod
+    def compute_depth(cls, depth, model_class, name='depth'):
+        try:
+            model_depth = getattr(model_class._meta, 'depth', getattr(model_class.Meta, 'depth', 10))
+            depth = min(depth, int(model_depth))
+        except AttributeError:
+            depth = min(depth, int(getattr(model_class._meta, 'depth', 1)))
+
+        return depth
+
     @classmethod
     def many_init(cls, *args, **kwargs):
         kwargs['child'] = cls(**kwargs)
         try:
-            cls.Meta.depth = kwargs['context']['view'].many_depth
+            cls.Meta.depth = cls.compute_depth(kwargs['context']['view'].many_depth, cls.Meta.model, 'many_depth')
         except KeyError:
             pass
         return ContainerSerializer(*args, **kwargs)
@@ -332,8 +346,13 @@ class LDPSerializer(HyperlinkedModelSerializer):
     def get_value(self, dictionary):
         try:
             object_list = dictionary["@graph"]
-            container_id = Model.get_container_path(self.parent.instance)
-            obj = next(filter(lambda o: container_id in o[self.url_field_name], object_list))
+            if self.parent.instance is None:
+                obj = next(filter(
+                    lambda o: not hasattr(o, self.parent.url_field_name) or "./" in o[self.url_field_name],
+                    object_list))
+            else:
+                container_id = Model.container_id(self.parent.instance)
+                obj = next(filter(lambda o: container_id in o[self.url_field_name], object_list))
             item = super().get_value(obj)
             full_item = None
             if item is empty:
@@ -358,12 +377,13 @@ class LDPSerializer(HyperlinkedModelSerializer):
         return instance
 
     def attach_related_object(self, instance, validated_data):
-        ModelClass = self.Meta.model
+        model_class = self.Meta.model
 
-        info = model_meta.get_field_info(ModelClass)
+        info = model_meta.get_field_info(model_class)
         many_to_many = {}
         for field_name, relation_info in info.relations.items():
-            if relation_info.to_many and relation_info.reverse and not (field_name in validated_data):
+            if relation_info.to_many and relation_info.reverse and not (
+                    field_name in validated_data) and not field_name is None:
                 rel = getattr(instance._meta.model, field_name).rel
                 if rel.name in validated_data:
                     related = validated_data[rel.name]
@@ -389,9 +409,11 @@ class LDPSerializer(HyperlinkedModelSerializer):
 
         for attr, value in validated_data.items():
             if isinstance(value, dict):
+                slug_field = Model.slug_field(instance)
                 manager = getattr(instance, attr)
-                if 'pk' in value:
-                    oldObj = manager._meta.model.objects.get(pk=value['pk'])
+                if slug_field in value:
+                    kwargs = {slug_field: value[slug_field]}
+                    oldObj = manager._meta.model.objects.get(**kwargs)
                     value = self.update(instance=oldObj, validated_data=value)
                 else:
                     value = self.internal_create(validated_data=value, model=manager._meta.model)
@@ -406,18 +428,20 @@ class LDPSerializer(HyperlinkedModelSerializer):
     def save_or_update_nested_list(self, instance, nested_fields):
         for (field_name, data) in nested_fields:
             manager = getattr(instance, field_name)
+            slug_field = Model.slug_field(instance)
 
-            item_pk_to_keep = list(map(lambda e: int(e['pk']), filter(lambda x: 'pk' in x, data)))
+            item_pk_to_keep = list(map(lambda e: e[slug_field], filter(lambda x: slug_field in x, data)))
             for item in list(manager.all()):
-                if not item.pk in item_pk_to_keep:
+                if not str(getattr(item, slug_field)) in item_pk_to_keep:
                     if getattr(manager, 'through', None) is None:
                         item.delete()
                     else:
                         manager.remove(item)
 
             for item in data:
-                if 'pk' in item:
-                    oldObj = manager.model.objects.get(pk=item['pk'])
+                if slug_field in item:
+                    kwargs = {slug_field: item[slug_field]}
+                    oldObj = manager.model.objects.get(**kwargs)
                     savedItem = self.update(instance=oldObj, validated_data=item)
                 else:
                     rel = getattr(instance._meta.model, field_name).rel
