@@ -1,12 +1,12 @@
 from django.apps import apps
 from django.conf import settings
 from django.conf.urls import url, include
+from django.contrib.auth.models import User
 from django.core.exceptions import FieldDoesNotExist
 from django.core.urlresolvers import get_resolver
-from django.db.utils import OperationalError
+from django.db.utils import OperationalError, ProgrammingError
 from django.shortcuts import get_object_or_404
 from django.utils.decorators import classonlymethod
-from djangoldp.models import LDPSource
 from guardian.shortcuts import get_objects_for_user
 from pyld import jsonld
 from rest_framework.authentication import SessionAuthentication
@@ -14,6 +14,7 @@ from rest_framework.parsers import JSONParser
 from rest_framework.renderers import JSONRenderer
 from rest_framework.viewsets import ModelViewSet
 
+from djangoldp.models import LDPSource, Model
 from .serializers import LDPSerializer
 
 
@@ -89,8 +90,8 @@ class LDPViewSet(LDPViewSetGenerator):
     """An automatically generated viewset that serves models following the Linked Data Platform convention"""
     fields = None
     exclude = None
-    depth = 1
-    many_depth = 0
+    depth = 2
+    many_depth = 1
     renderer_classes = (JSONLDRenderer,)
     parser_classes = (JSONLDParser,)
     authentication_classes = (NoCSRFAuthentication,)
@@ -119,7 +120,7 @@ class LDPViewSet(LDPViewSetGenerator):
         return type(LDPSerializer)(model_name + 'Serializer', (LDPSerializer,), {'Meta': meta_class})
 
     def perform_create(self, serializer, **kwargs):
-        if hasattr(self.model._meta, 'auto_author'):
+        if hasattr(self.model._meta, 'auto_author') and isinstance(self.request.user, User):
             kwargs[self.model._meta.auto_author] = self.request.user
         serializer.save(**kwargs)
 
@@ -194,7 +195,8 @@ class LDPNestedViewSet(LDPViewSet):
             related_field=related_field,
             parent_lookup_field=cls.get_lookup_arg(**kwargs),
             model_prefix=cls.get_model(**kwargs)._meta.object_name.lower(),
-            permission_classes=kwargs.get('permission_classes', ()),
+            permission_classes=Model.get_permission_classes(related_field.related_model,
+                                                            kwargs.get('permission_classes', ())),
             lookup_url_kwarg=related_field.related_model._meta.object_name.lower() + '_id')
 
 
@@ -207,7 +209,7 @@ class LDPSourceViewSet(LDPViewSet):
         try:
             return include([url(name + '/', super(LDPSourceViewSet, cls).urls(federation=name, **kwargs))
                             for name in LDPSource.objects.order_by().values_list('federation', flat=True).distinct()])
-        except OperationalError:  # for the case where the table doesn't exist
+        except (OperationalError, ProgrammingError):  # for the case where the table doesn't exist
             return include([])
 
     def get_queryset(self, *args, **kwargs):
